(***********************************************************************)
(*                                                                     *)
(*                              My_stdlib                              *)
(*                                                                     *)
(*             projets Estime & Cristal, INRIA-Rocquencourt            *)
(*                                                                     *)
(*  Copyright 2004-2006 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: options.ml,v 1.1 2007/01/17 23:36:00 fclement Exp $ *)

type option_name = string;;
type manual_text = string;;

let debug_actions = ref []
and report_actions = ref [];;

let launch_debug_actions () =
  List.iter (fun action -> action ()) !debug_actions
and launch_report_actions () =
  List.iter (fun action -> action ()) !report_actions;;

let options =
  ref [
    "--debug", Arg.Unit launch_debug_actions, "  Turn on all declared debug options.";
    "-d", Arg.Unit launch_debug_actions, "  Same as --debug.\n";
    "--report", Arg.Unit launch_report_actions, "  Turn on all declared report options.";
    "-r", Arg.Unit launch_report_actions, "  Same as --report.\n";
  ];;

let all () = !options;;

let add oname action man =
  options := (oname, action, man) :: !options;;

let add_all opts =
  List.iter (fun (oname, action, man) -> add oname action man) opts;;

let add_debug oname action man =
  add oname action man;
  match action with
  | Arg.Unit action -> debug_actions := action :: !debug_actions
  | _ -> invalid_arg "add_debug: not a unit argument";;

let add_all_debug opts =
  List.iter (fun (oname, action, man) -> add_debug oname action man) opts;;

let add_report oname action man =
  add oname action man;
  match action with
  | Arg.Unit action -> report_actions := action :: !report_actions
  | _ -> invalid_arg "add_report: not a unit argument";;

let add_all_report opts =
  List.iter (fun (oname, action, man) -> add_report oname action man) opts;;

(* A special case: flag options. *)
let flag init oname man =
  let r = ref init in
  add oname (if init then Arg.Clear r else Arg.Set r) man;
  r;;

(* Another special case: debug options (init value is false). *)
let debug oname man =
  let r = ref false in
  add oname (Arg.Set r) man;
  (fun s -> if !r then prerr_endline s);;
